@@ -18,10 +18,6 @@ except ImportError:
     from asyncio import get_event_loop as get_running_loop
 
 
-class InotifyError(RuntimeError):
-    '''Simple inotify error type, thrown for all errors in inotify ffi calls'''
-
-
 # Suppress things that might break documentation generation
 if os.environ.get('ASYNCINOTIFY_DO_NOT_IMPORT') != 'TRUE':
     from ._ffi import libc, inotify_event, inotify_event_size, NAME_MAX
@@ -387,7 +383,7 @@ class Inotify:
     @property
     def fd(self) -> int:
         if self._fd is None:
-            raise RuntimeError('Can not work with closed inotify')
+            raise ValueError('Can not work with closed inotify')
         else:
             return self._fd
 
@@ -403,14 +399,18 @@ class Inotify:
         # Convert bytes to Path
         if isinstance(path, bytes):
             bytepath = path
-            path = Path(path.decode('utf-8', 'surrogateescape'))
+            path = Path(os.fsdecode(bytepath))
         else:
             # Convert non-Path to Path
             if not isinstance(path, Path):
                 path = Path(path)
-            bytepath = path.__fspath__().encode('utf-8', 'surrogateescape')
+            bytepath = bytes(path)
 
-        wd = libc.inotify_add_watch(self.fd, bytepath, mask)
+        try:
+            wd = libc.inotify_add_watch(self.fd, bytepath, mask)
+        except OSError as e:
+            e.filename = path
+            raise e
 
         # Happens for things like an existing watch instance being modified,
         # like MASK_ADD
@@ -498,7 +498,7 @@ class Inotify:
                     # If zero_pos is -1, we want the whole name string, otherwise truncate the zeros
                     if zero_pos > 0:
                         raw_name = raw_name[:zero_pos]
-                    name = Path(raw_name.decode('utf-8', 'surrogateescape'))
+                    name = Path(os.fsdecode(raw_name))
             mask = Mask(event_struct.mask)
 
             watch: Optional[Union[Watch, ReferenceType]] = self._watches.get(event_struct.wd, None)
